import requests
import datetime
from slugify import slugify


class TeamClient(object):

    def __init__(self, api_url, team_name):
        self.api_url = api_url
        self.team_name = team_name

    def slugify(self, name):
        return slugify(
            name,
            regex_pattern=r'[^\w\S-]'
        )

    @property
    def team(self):
        if not self.api_url:
            raise Exception('Team API URL not informed.')
        if not self.team_name:
            raise Exception('Team name not informed.')
        slugify_name = self.slugify(self.team_name)
        url = '{}/slug/{}'.format(self.api_url, slugify_name)
        res = requests.get(url)
        if res.ok:
            return res.json()
        error = 'Team {} not found.'.format(self.team_name)
        raise Exception(error)

    @property
    def team_id(self):
        return self.team.get('id')

    def make_labels(self, engine_name='', infra_name='', database_name=''):

        team = self.team

        labels = {
            'servico_de_negocio': team.get('servico-de-negocio'),
            'cliente': team.get('cliente'),
            'team_slug_name': team.get('slug'),
            'team_id': team.get('id'),
            'created_at': datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'),
            'engine': engine_name,
            'infra_name': infra_name,
            'database_name': database_name,
            'origin': 'dbaas'
        }

        return labels



from time import sleep
from socket import timeout

from .base import BaseProviderObject
from .base import MongoClient
import logging

class BaseProvider(BaseProviderObject):
    SECONDS_RETRY = 3
    MAX_RETRY = 300

    def __init__(self, environment, engine=None, auth_info=None):
        super(BaseProvider, self).__init__()
        self.environment = environment
        self._client = None
        self._credential = None
        self._commands = None
        self.engine = engine
        self.auth_info = auth_info

    @property
    def client(self):
        if not self._client:
            self._client = self.build_client()
        return self._client

    @property
    def credential(self):
        if not self._credential:
            self._credential = self.build_credential()
        return self._credential

    def credential_add(self, content):
        credential_cls = self.get_credential_add()
        credential = credential_cls(self.provider, self.environment, content)

        is_valid, error = credential.is_valid(content)
        if not is_valid:
            return False, error

        try:
            insert = credential.save()
        except Exception as e:
            return False, str(e)
        else:
            return True, insert.get('_id')

    def build_client(self):
        raise NotImplementedError

    def build_credential(self):
        raise NotImplementedError

    def get_credential_add(self):
        raise NotImplementedError

    def get_or_none_resource(self, operation, **kw):
        try:
            response = operation().get(**kw).execute()
        except Exception as ex:
            if ex.resp.status == 404:
                return None
            raise Exception(ex)

        return response

    @classmethod
    def get_provider(cls):
        raise NotImplementedError

    @property
    def provider(self):
        return self.get_provider()

    def wait_operation(self, operation=None, region=None, zone=None):
        op = self._wait(
            operation=operation,
            region=region,
            zone=zone
        )
        return self._check_operation_status(op)

    def _wait(self, operation, region=None, zone=None):
        if not operation:
            raise EnvironmentError('operation must be provided')

        retry = 0
        if zone:
            op = self._get_wait_zone_operation(
                zone=zone,
                operation=operation
            )
        elif region:
            op = self._get_wait_region_operation(
                region=region,
                operation=operation
            )
        else:
            op = self._get_wait_global_operation(
                operation=operation
            )

        while retry <= self.MAX_RETRY:
            attempt = 'Attempt {} of {}.'.format(retry, self.MAX_RETRY)
            try:
                operation = op.execute()
            except Exception as ex:
                if isinstance(ex, timeout):
                    err = 'Timeout waiting for operation. {}.'.format(attempt)
                    logging.error(err)
                    retry += 1
                    sleep(self.SECONDS_RETRY)
                    continue
                else:
                    logging.error('Exception executing operation.')
                    raise Exception(ex)
            else:
                if operation.get('status') in ('PENDING', 'RUNNING'):
                    err = 'Operation [{}] is still {}. {}.'.format(
                        operation, operation.get('status'), attempt
                    )
                    logging.error(err)
                    retry += 1
                    sleep(self.SECONDS_RETRY)
                    continue
                return operation

        raise EnvironmentError('Error while wait %s operation' % operation)

    def _get_wait_zone_operation(self, zone, operation, execute_request=False):
        operation = self.client.zoneOperations().wait(
            project=self.credential.project,
            zone=zone,
            operation=operation
        )

        if execute_request:
            return operation.execute()

        return operation

    def _get_wait_region_operation(self, region, operation,
                                   execute_request=False):
        operation = self.client.regionOperations().wait(
            project=self.credential.project,
            region=region,
            operation=operation
        )

        if execute_request:
            return operation.execute()

        return operation

    def _get_wait_global_operation(self, operation, execute_request=False):
        operation = self.client.globalOperations().wait(
            project=self.credential.project,
            operation=operation
        )

        if execute_request:
            return operation.execute()

        return operation

    def _check_operation_status(self, operation):
        if operation.get('error'):
            error = 'Error in {} operation: {}'.format(
                operation.get('operationType'),
                operation.get('error')
            )
            raise Exception(error)

        if operation.get('status') != 'DONE':
            error = 'Operation {} is not Done. Status: {}'.format(
                operation.get('operationType'),
                operation.get('status')
            )
            raise Exception(error)

        return True



class Version(object):
    """Version of the package"""

    def __setattr__(self, *args):
        raise TypeError("can't modify immutable instance")
    __delattr__ = __setattr__

    def __init__(self, num):
        super(Version, self).__setattr__('number', num)



from pymongo import MongoClient, ReturnDocument


class BaseProviderObject(object):
    _provider_type = None

    def __init__(self, *args, **kwargs):
        pass

    @property
    def provider_type(self):
        if self._provider_type:
            return self._provider_type
        raise NotImplementedError

    @provider_type.setter
    def provider_type(self, prov_type):
        self._provider_type = prov_type



from collections import OrderedDict

from .base import BaseProviderObject
from .base import MongoClient


class BaseCredential(BaseProviderObject):

    MONGODB_PARAMS = None
    MONGODB_DB = None
    MONGO_ENDPOINT = None
    MONGODB_HOST = None
    MONGODB_USER = None
    MONGODB_PORT = None
    MONGODB_PWD = None
    MONGODB_DB = None
    provider_type = None

    def __init__(self, provider, environment):
        super(BaseCredential, self).__init__()
        self.provider = provider
        self.environment = environment
        self._db = None
        self._collection_credential = None
        self._content = None
        self._zone = None

    def __getattribute__(self, name):
        try:
            attr = object.__getattribute__(self, name)
        except AttributeError as e:
            attr = self.content.get(name)
            if attr is None:
                raise AttributeError(e)
        return attr

    @property
    def db(self):
        if not self._db:
            if self.provider_type == "host_provider":
                params = {'document_class': OrderedDict}
                if self.MONGO_ENDPOINT is None:
                    params.update({
                        'host': self.MONGODB_HOST,
                        'port': self.MONGODB_PORT,
                        'username': self.MONGODB_USER,
                        'password': self.MONGODB_PWD
                    })
                    client = MongoClient(**params)
                else:
                    client = MongoClient(self.MONGO_ENDPOINT, **params)
                self._db = client[self.MONGODB_DB]
            else:
                client = MongoClient(**self.MONGODB_PARAMS)
                self._db = client[self.MONGODB_DB]

        return self._db

    @property
    def credential(self):
        if not self._collection_credential:
            self._collection_credential = self.db[self._credential_idx]
        return self._collection_credential

    @property
    def _credential_idx(self):
        return ("credential"
                if self.provider_type == "host_provider"
                else "credentials")

    @property
    def content(self):
        if not self._content:
            raise NotImplementedError
        return self._content



import setuptools
from dbaas_base_provider.version import Version


setuptools.setup(name='dbaas-base-provider',
                 version=Version('0.0.27').number,
                 description='Base for DBaaS providers',
                 long_description=open('README.md').read().strip(),
                 author='William Marquardt',
                 author_email='williammqt@gmail.com',
                 packages=[
                    'dbaas_base_provider',
                 ],
                 package_dir={
                     'dbaas_base_provider': 'dbaas_base_provider'
                 },
                 url='https://github.com/bento-dbaas/dbaas-base-provider',
                 py_modules=['dbaas_base_provider'],
                 install_requires=[
                    'mongoengine>=0.15.0',
                    'pymongo==3.6.1',
                    'python-slugify==4.0.1'
                 ],
                 license='BSD 3-Clause License',
                 zip_safe=False,
                 keywords='dbaas',
                 classifiers=['Development Status :: 5 - Production/Stable'])



import logging
from functools import wraps
from datetime import datetime
import uuid
from flask import request


def log_this(f):
    """
    IMPORTANT: If you use this decorator with Flask, be sure to put it
    it after @route (Flask demands this to be the top decorarator)
    https://flask.palletsprojects.com/en/2.0.x/patterns/viewdecorators/
    """

    @wraps(f)
    def fw(*args, **kwargs):
        id = uuid.uuid4()
        logging.info("#" * 80)
        logging.info("{} | Start: {} - {}".format(f.__name__, datetime.now(), id))
        logging.info("args: {} - {}".format(args, id))
        logging.info("kwargs: {} - {}".format(kwargs, id))
        logging.info("json data: {} - {}".format(request.get_json(), id))
        retFunc = f(*args, **kwargs)
        logging.info("{} | End: {} - {}".format(f.__name__, datetime.now(), id))
        logging.info("#" * 80)
        return retFunc

    return fw
